@@ -42,6 +42,10 @@ from .ppss_auth_utils.password import getPasswordDigest
 l = logging.getLogger('ppssauth.models')
 
 
+class constants():
+    SYSADMINPERM = "sysadmin"
+    SYSADMINGROUP = "sysadmin"
+
 def __createPermissions(session,permissions):
     permmap = {}
     for p in permissions:
@@ -95,7 +99,7 @@ def __createUsers(session,users,groups,defaultpassword = None):
             session.add(user)
         usermap[uname] = user
         for g in grouplist:
-            if not user.isInGroup(g):
+            if not user.isInGroup(g,False):
                 if g in groups:
                     group = groups[g]
                 else:
@@ -109,11 +113,6 @@ def __createUsers(session,users,groups,defaultpassword = None):
 def initdb(session=None,createdefault=False,engine = None):
     if engine is None:
         engine = session.get_bind()
-
-    #Base.metadata.create_all( engine )
-    #if session is None:
-    #    return 0
-    #DBVersion.updateDB(session)
     try:
         PPSsuser.all(session)
     except:
@@ -124,20 +123,20 @@ def initdb(session=None,createdefault=False,engine = None):
                 {"name": u"edituser","permtype": 1},
                 {"name": u"listuser","permtype": 1},
                 {"name": u"login"   ,"permtype": 1},
-                {"name": u"sysadmin","permtype": 1},
+                {"name": constants.SYSADMINPERM,"permtype": 1},
         ]
         permissions = [{'name':p,'permtype':1} for p in Conf.perm2create]
         for sp in systemperms:
             permissions.append(sp)
 
         permissionmap = __createPermissions(session,permissions)
-        groupmap = __createGroups(session, Conf.group2create + ["useradmin=edituser,listuser,login,sysadmin"] , permissionmap)
+        l.debug("permission map:{}".format(permissionmap))
+        groupmap = __createGroups(session, Conf.group2create + ["useradmin=edituser,listuser,login","{}={}".format(constants.SYSADMINGROUP,constants.SYSADMINPERM)] , permissionmap)
+        l.debug("group map:{}".format(groupmap))
         if Conf.adminname:
-            root = __createUsers(session,[Conf.adminname+"="+u"sysadmin"],groupmap,Conf.adminpass if Conf.adminpass else None)
-            #if Conf.adminpass:
-            #    root[Conf.adminname].setPassword(Conf.adminpass,False)
+            root = __createUsers(session,[Conf.adminname+"="+constants.SYSADMINGROUP],groupmap,Conf.adminpass if Conf.adminpass else None)
         usermap = __createUsers(session, Conf.user2create, groupmap )
-        
+        l.debug("user map:{}".format(usermap))
         return 0
     
     return 0
@@ -211,6 +210,12 @@ class PPSsuser(Base,commonTable):
     groups = relationship("PPSsgroup",secondary=ppssuserlkppssgroup,lazy='joined',cascade="all", 
         backref=backref('users',lazy='select',order_by='PPSsuser.username'))
 
+
+    superusercache = None
+    permissionscache = None
+    permissionsmapcache = None
+    groupmapcache = None
+
     @classmethod
     def checkLogin(cls,user,password,dbsession):
         return PPSsuser.checkCryptedLogin(user,getPasswordDigest(password),dbsession)
@@ -235,45 +240,59 @@ class PPSsuser(Base,commonTable):
         return { "id": self.id, "username":self.username,"enabled":self.enabled}
 
 
-
     def setPassword(self,password,canexpire = True):
-        
         dig = getPasswordDigest(password)
         self.password= dig
-        if Conf.passwordexpire:
+        if Conf.passwordexpire and canexpire:
             self.passwordexpire = datetime.now() + timedelta(days = Conf.passwordexpire)
         self.passowrdhistory.append( PPSspasswordhistory(password = dig)  )
         return self
 
+
     def getPermissions(self):
-        result = set()
-        for g in self.groups:
-            if g.enabled:
-                for p in g.permissions:
-                    result.add( (p.id, p.name, p.permtype) )
-        return list(set(result))
+        if self.permissionscache is None:
+            result = set()
+            for g in self.groups:
+                if g.enabled:
+                    for p in g.permissions:
+                        result.add( (p.id, p.name, p.permtype) )
+            self.permissionscache = list(set(result))
+        return self.permissionscache
         # return set([p.name for p in [g.permissions for g in self.groups if g.enabled]] )
 
+    def getPermissionsMap(self):
+        if self.permissionsmapcache is None:
+            permlist = self.getPermissions()
+            self.permissionsmapcache = set(permlist)
+        return self.permissionsmapcache
+
     def isSuperUser(self):
-        if self.username == Conf.adminname:return True
-        for p in self.getPermissions():
-            if p[2]==1 and (p[1] == u'superadmin' or p[1] == u'sysadmin'):
-                return True
-        return False
+        if self.superusercache is None:
+            self.superusercache = False
+            if self.username == Conf.adminname:return True
+            for p in self.getPermissions():
+                if p[2]==1 and (p[1] == u'superadmin' or p[1] == u'sysadmin'):
+                    self.superusercache = True
+                    break
+        return self.superusercache
 
     def hasPermission(self, permission):
-        for p in self.getPermissions():
-            if p[2]==1 and p[1] == permission:
-                return True
-        return False
+        if self.isSuperUser():
+            return True
+        return permission in self.getPermissionsMap()
+
+    def getGroupsMap(self):
+        if self.groupmapcache is None:
+            grouplist = []
+            for g in self.groups:
+                grouplist.append(g.name)
+            self.groupmapcache = set(grouplist)
+        return self.groupmapcache
 
 
-    def isInGroup(self,groupname):
-        if self.username == Conf.adminname:return True
-        for g in self.groups:
-            if g.name == groupname:
-                return True
-        return False
+    def isInGroup(self,groupname,whitelie=True):
+        if self.isSuperUser() and whitelie:return True
+        return groupname in self.getGroupsMap()
 
     def __unicode__(self):
         return u"<PPSsuser ({id}-{name},{enabled})>".format(id=self.id,name=self.username, enabled=self.enabled)
@@ -305,6 +324,9 @@ class PPSsgroup(Base,commonTable):
     enabled= Column(Integer,default=1)
     permissions = relationship("PPSspermission",cascade="all",secondary=ppssgrouplkppsspermission  ,backref=backref('groups'))
 
+    permissionsmapcache = None
+    permissionscache = None
+
     def __unicode__(self):
         return u"<PPSsgroup {name} ({id})>".format(name=self.name,id=self.id)
     def __str__(self):
@@ -319,11 +341,16 @@ class PPSsgroup(Base,commonTable):
     def permdict(self):
         return [x.todict() for x in self.permissions]
 
+    def getPermissionsMap(self):
+        if self.permissionsmapcache is None:
+            self.permissionscache = []
+            for p in self.permissions:
+                self.permissionscache.append(p.name)
+            self.permissionsmapcache = set(self.permissionscache)
+        return self.permissionsmapcache
+
     def hasPermission(self,permissionname):
-        for p in self.permissions:
-            if p.name == permissionname:
-                return True
-        return False
+        return permissionname in self.getPermissionsMap()
 
     @classmethod
     def byName(cls,name,dbsession):
