@@ -6,7 +6,7 @@ from pyramid.settings import asbool
 from pyramid.renderers import render_to_response
 
 from ..constants import Conf
-from ..models import PPSsuser,PPSsgroup,PPSspermission
+from ..models import PPSsuser,PPSsgroup,PPSspermission,constants
 from ..ppss_auth_utils import checkPassword
 
 from pyramid.view import view_config,forbidden_view_config
@@ -32,7 +32,9 @@ def getPrincipals(uid,request):
     l.debug("####  usergroups:{g}".format(g=groups))
     return groups
 
-
+#this class build the ACL consumed by ACLRoot
+#ACL is in the form: {group name: [list of permission names]} and is derived by ppss_groups and ppss_permission)
+#permissions to be used in views are ppss_permission elements.
 class ACLBuilder(object):
     def __init__(self,baseACL,dbsession):
         self.baseACL = baseACL
@@ -48,39 +50,22 @@ class ACLBuilder(object):
                     tuple([str(p.name) for p in group.permissions])  
             ) )
         acl = self.baseACL + acl
-        l.debug("acl built: {acl}".format(acl = acl))
+        l.warn("ACLBuilder:acl built: {acl}".format(acl = acl))
         return acl
 
 
-
+#This class stores the acl structure used by the root factory
 class ACLRoot(object):
     baseACL=[(Allow, Authenticated, 'view'),
-        (Allow, 'g:superadmin', ALL_PERMISSIONS),
-        (Deny,  'g:sysadmin', 'deleteadmin'),
-        (Allow, 'g:sysadmin', ALL_PERMISSIONS)
+        (Allow, 'g:'+constants.SYSADMINGROUP, ALL_PERMISSIONS)
         ]
 
     lastupdateACL = datetime.datetime.now()
     __acl__ = [
-        (Allow, 'g:superadmin', ALL_PERMISSIONS),
-        (Allow, 'g:sysadmin', ALL_PERMISSIONS),
+        (Allow, 'g:'+constants.SYSADMINPERM, ALL_PERMISSIONS)
     ]
 
 
-    @classmethod
-    @cache_region('short_term', 'ACLpermissions')
-    def buildACL(cls,dbsession):
-        groups = dbsession.query(PPSsgroup).filter(PPSsgroup.enabled==1).all()
-        acl = [] 
-        for group in groups:
-            acl.append( (Allow,
-                    str("g:"+group.name),
-                    tuple([str(p.name) for p in group.permissions])  
-            ) )
-        ACLRoot.__acl__ = ACLRoot.baseACL + acl
-        l.debug("built ACL:{acl}".format(acl=repr(ACLRoot.__acl__)) )
-
-
     def __init__(self, request):
         self.request = request
         aclb = ACLBuilder(ACLRoot.baseACL,request.dbsession)
@@ -132,7 +117,7 @@ class AuthController():
     def login(self):
         r = self.request
         postloginpage = self.request.referer if (self.request.referer and self.request.referer!=self.request.route_url("ppsslogin")) else r.route_url(Conf.postloginroute)
-        l.info("plp = {},referer = {}, loginurl = {}, plr = {}, is true? {}".format(
+        l.info("postloginpage = {},referer = {}, loginurl = {}, plr = {}, is true? {}".format(
             postloginpage, 
             self.request.referer, 
             self.request.route_url("ppsslogin"),
@@ -147,7 +132,7 @@ class AuthController():
             res = None
             l.info("Login attempt: u={username}".format(username=username))
             if username == Conf.adminname:
-                superuser =True
+                superuser = True
                 if Conf.adminpass and password == Conf.adminpass:
                     u = PPSsuser.byField('username',username,r.dbsession)
                     if u:
@@ -162,7 +147,7 @@ class AuthController():
                 if superuser:
                     l.info("{username} logged in as superuser".format(username=username) )
                     r.session['admin'] = True
-                    r.session['principals'] = ["g:admin","g:superadmin"]
+                    r.session['principals'] = ["g:admin","g:sysadmin"]
                 else:
                     l.debug("{username} logged in as normal user".format(username=username) )
                     r.session['principals'] = res.getPrincipals(res.passwordExpired()) 
